import org.joone.engine.*;
import org.joone.net.*;
import lejos.pc.comm.*;
import java.io.*;

public class XorNet extends Thread {

	NeuralNet nnet = null;

	DirectSynapse memInp = null;
	DirectSynapse memOut = null;

	NXTComm nxtComm = null;
	DataOutputStream dos = null;
	DataInputStream dis = null;
	boolean running = true;

	public XorNet(String snet) {

		NeuralNetLoader netLoader = new NeuralNetLoader(snet);
		nnet = netLoader.getNeuralNet();

		Layer input = nnet.getInputLayer();
		input.removeAllInputs();
		memInp = new DirectSynapse();
		input.addInputSynapse(memInp);

		Layer output = nnet.getOutputLayer();
		output.removeAllOutputs();
		memOut = new DirectSynapse();
		output.addOutputSynapse(memOut);

		nnet.getMonitor().setLearning(false);

		nnet.start();

	}

	public void netTest() {
		Pattern iPattern = new Pattern(new double[] { 1, 0 });
		memInp.fwdPut(iPattern);
		Pattern oPattern = memOut.fwdGet();
		System.out.println("netTest: Output Pattern = " + oPattern.getArray()[0]);
	}

	public void startNXT(String name, String address) {

		nxtComm = NXTCommFactory.createNXTComm(NXTCommFactory.BLUETOOTH);

		NXTInfo[] nxtInfo = new NXTInfo[1];

		nxtInfo[0] = new NXTInfo(name, address);

		System.out.println("Connecting to " + nxtInfo[0].btResourceString);

		boolean opened = nxtComm.open(nxtInfo[0]);

		if (!opened) {
			System.out.println("Failed to open " + nxtInfo[0].name);
			System.exit(1);
		}

		System.out.println("Connected to " + nxtInfo[0].btResourceString);

		InputStream is = nxtComm.getInputStream();
		OutputStream os = nxtComm.getOutputStream();

		dos = new DataOutputStream(os);
		dis = new DataInputStream(is);

	}

	public void dataReceived(int x1, int x2) {
		if (dos == null)
			return;

		Pattern iPattern = new Pattern(new double[] { x1, x2 });
		memInp.fwdPut(iPattern);
		Pattern oPattern = memOut.fwdGet();
		int out = (oPattern.getArray()[0] > 0.5) ? 1 : 0;
		System.out.println("Output Pattern = " + out);

		try {
			dos.writeInt(out);
			dos.flush();
		} catch (IOException ioe) {
			System.out.println("IO Exception writing bytes");
		}

	}


	public void run() {
		while (running && dis != null) {
			try {
				int sensorIn = dis.readInt();
				System.out.println("SensorIn = " + sensorIn);

				if (sensorIn == 9) {
					suicide();
				} else {
					int x1 = (sensorIn & 2) >> 1;
					int x2 = sensorIn & 1;
					dataReceived(x1, x2);
				}

			} catch (IOException ioe) {
				System.out.println("IO Exception reading bytes");
			}
		}
	}

	public void suicide() {

		try {
			if (dis != null)
				dis.close();
			// if(dos!=null) dos.close(); Why does this prevent re-connection?
			if (nxtComm != null)
				nxtComm.close();
		} catch (IOException ioe) {
			System.out.println("IOException closing connection");
		}

		if (nnet != null)
			nnet.stop();
		running = false;
	}



	/**
	 * @param args
	 */
	public static void main(String[] args) {
		XorNet xornet = new XorNet("d:/xor.snet");
		xornet.netTest();
		xornet.startNXT("NXT", "00:16:53:05:4C:05");
		xornet.start();
	}
}
